# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, List, Literal


class Subscription(TypedDict, total=False):
    """Previous billing date of the subscription."""
    previous_billing_at: Optional[str]
    """End date of the trial period."""
    trial_period_end: Optional[str]
    """Start date of the trial period."""
    trial_period_start: Optional[str]
    """Date the subscription was canceled."""
    canceled_at: Optional[str]
    """Payment collection method for invoices generated by the subscription."""
    collection_method: Optional[str]
    """List of items in the subscription."""
    items: Optional[List[dict]]
    """Plan ID associated with the subscription."""
    plan_id: Optional[str]
    """Currency of the subscription."""
    currency: Optional[str]
    """First billing date of the subscription."""
    first_billing_at: Optional[str]
    """Status of the subscription."""
    status: Optional[Literal["draft", "pending", "trialing", "active", "paused", "canceled", "expired"]]
    """Timestamp indicating when the object was created."""
    created_at: Optional[str]
    """Start date of the current billing period."""
    current_period_start: Optional[str]
    """Represents the object's type."""
    object: Optional[Literal["subscription"]]
    """ID of the object."""
    id: Optional[str]
    """Date the subscription is billed through."""
    billed_through: Optional[str]
    """End date of the current billing period."""
    current_period_end: Optional[str]
    """Date the subscription started."""
    started_at: Optional[str]
    """Billingrails account ID associated with the subscription."""
    account_id: Optional[str]
    """Billing cycle type of the subscription."""
    billing_cycle_type: Optional[Literal["anniversary", "calendar"]]
    """Payment method ID associated with the subscription."""
    payment_method_id: Optional[str]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, Literal


class PaymentRequest(TypedDict, total=False):
    """ID of the payment request."""
    id: Optional[str]
    """Number of days until the payment request is due."""
    due_in_days: Optional[int]
    """Represents the object's type."""
    object: Optional[Literal["payment_request"]]
    """Type of payment request."""
    type: Optional[str]
    """Order of this payment request."""
    ordinal: Optional[int]
    """Status of the payment request."""
    status: Optional[str]
    """Date the payment request is due."""
    due_at: Optional[str]
    """Amount due in the payment request."""
    due_amount: Optional[int]
    """Currency of the payment request."""
    currency: Optional[str]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, List

from .product import Product


class ProductListResponse(TypedDict, total=False):
    products: Optional[List[Product]]
    meta: Optional[dict]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, Literal


class PaymentPage(TypedDict, total=False):
    """Type of payment page."""
    type: Optional[Literal["subscription"]]
    """ID of the payment page."""
    id: Optional[str]
    """URL to redirect to after payment completion."""
    return_url: Optional[str]
    """Name of the payment page."""
    name: Optional[str]
    """URL-friendly identifier for the payment page."""
    slug: Optional[str]
    """Description of the payment page."""
    description: Optional[str]
    """Represents the object's type."""
    object: Optional[Literal["payment_page"]]
    """Public URL for the payment page."""
    hosted_url: Optional[str]
    """Status of the payment page."""
    status: Optional[Literal["draft", "active", "archived"]]
    """Timestamp indicating when the object was created."""
    created_at: Optional[str]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, List, Literal

from .price import Price
from .interval import Interval


class FeeInput(TypedDict, total=False):
    """Feature entitlements associated with this fee."""
    entitlements: Optional[List[dict]]
    """Meter profile ID to use for this fee."""
    meter_profile_id: Optional[str]
    """Name that appears on invoices for this fee."""
    invoice_name: Optional[str]
    """Unique code for the fee."""
    code: Optional[str]
    """When to bill for this fee."""
    bill_timing: Optional[Literal["advance", "arrears"]]
    """Number of billing cycles for this fee."""
    billing_cycles: Optional[int]
    """Plan ID or code associated with the fee."""
    plan_id: Optional[str]
    """Number of free units included."""
    free_units: Optional[int]
    """Billing interval for the fee."""
    interval: Optional[Interval]
    """Meter ID for usage-based billing."""
    meter_id: Optional[str]
    """Name of the fee."""
    name: Optional[str]
    """Description of the fee."""
    description: Optional[str]
    """Price configuration for the fee."""
    price: Optional[Price]



# This file is auto-generated. Do not edit manually.

from ..types import (
    ProductCreate,
    ProductListResponse,
    ProductResponse,
    ProductUpdate,
)


class ProductsResource:
    """Products resource"""

    def __init__(self, client):
        self.client = client

    def list(self, **params) -> ProductListResponse:
        """List products
        
        Retrieves a list of products."""
        return self.client.request("GET", f"/seller/products", params=params)

    def create(self, data: ProductCreate) -> ProductResponse:
        """Create a product
        
        Creates a product."""
        return self.client.request("POST", f"/seller/products", json=data)

    def retrieve(self, id: str, **params) -> ProductResponse:
        """Retrieve a product
        
        Retrieves a product by ID."""
        return self.client.request("GET", f"/seller/products/{id}", params=params)

    def update(self, id: str, data: ProductUpdate) -> ProductResponse:
        """Update a product
        
        Updates a product."""
        return self.client.request("PUT", f"/seller/products/{id}", json=data)



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional


class Address(TypedDict, total=False):
    """State."""
    state: Optional[str]
    """Country."""
    country: Optional[str]
    """Region."""
    region: Optional[str]
    """City."""
    city: Optional[str]
    """Address line 1."""
    line1: Optional[str]
    """Address line 2."""
    line2: Optional[str]
    """Postal code."""
    postal_code: Optional[str]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional

from .order import Order


class OrderResponse(TypedDict, total=False):
    order: Optional[Order]
    meta: Optional[dict]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, Literal


class Discount(TypedDict, total=False):
    """Maximum number of redemptions across all accounts."""
    max_redemptions: Optional[int]
    """Date when the discount expires."""
    valid_until: Optional[str]
    """Represents the object's type."""
    object: Optional[Literal["discount"]]
    """Maximum number of redemptions per account."""
    max_redemptions_per_account: Optional[int]
    """Percentage off."""
    percent_off: Optional[int]
    """Date when the discount becomes valid."""
    valid_from: Optional[str]
    """Account ID (if discount is account-specific)."""
    account_id: Optional[str]
    """Currency for fixed amount discounts."""
    currency: Optional[str]
    """Total number of redemptions across all accounts."""
    redemptions: Optional[int]
    """Amount off in currency subunits."""
    amount_off: Optional[int]
    """Unique code for the discount."""
    code: Optional[str]
    """Type of discount."""
    type: Optional[Literal["percentage", "fixed_amount"]]
    """Date when the discount expired."""
    expired_at: Optional[str]
    """Maximum number of billing periods discount can recur (null = forever)."""
    max_recurring_intervals: Optional[int]
    """Whether discount can be applied to multiple billing periods."""
    recurring: Optional[bool]
    """Name of the discount."""
    name: Optional[str]
    """Status of the discount."""
    status: Optional[Literal["pending", "active", "expired", "archived"]]
    """Timestamp indicating when the object was created."""
    created_at: Optional[str]
    """Description of the discount."""
    description: Optional[str]
    """ID of the discount."""
    id: Optional[str]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, Literal


class PaymentPageCreate(TypedDict, total=False):
    """Name of the payment page."""
    name: str
    """Status of the payment page."""
    status: Optional[Literal["draft", "active", "archived"]]
    """ID of the plan associated with the payment page. Required if type is `subscription`."""
    plan_id: Optional[str]
    """Type of payment page."""
    type: Literal["subscription"]
    """URL-friendly identifier for the payment page."""
    slug: Optional[str]
    """Unique code for the payment page."""
    code: str
    """Description of the payment page."""
    description: Optional[str]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, List, Literal

from .invoice_line_item import InvoiceLineItem
from .payment_request import PaymentRequest


class Invoice(TypedDict, total=False):
    """Subscription ID associated with the invoice."""
    subscription_id: Optional[str]
    """Total amount of the invoice including taxes, credits and discounts."""
    total_amount: Optional[int]
    """Credit amount applied to the invoice."""
    credit_amount: Optional[int]
    """URL to the hosted page of the invoice."""
    hosted_url: Optional[str]
    """Date the invoice is due."""
    due_at: Optional[str]
    """Date in the future when the invoice will be issued."""
    issue_at: Optional[str]
    """End date for the billing period."""
    billing_end: Optional[str]
    """Start date for the billing period."""
    billing_start: Optional[str]
    """Timestamp indicating when the object was created."""
    created_at: Optional[str]
    """Payment requests associated with this invoice."""
    payment_requests: Optional[List[PaymentRequest]]
    """Outstanding amount of the invoice."""
    outstanding_amount: Optional[int]
    """Status of the invoice."""
    status: Optional[Literal["pending", "draft", "issued", "overdue", "paid", "partially_paid", "refunded", "partially_refunded", "voided", "canceled", "failed"]]
    """Currency of the invoice."""
    currency: Optional[str]
    """Account ID associated with the invoice."""
    account_id: Optional[str]
    """Subtotal amount of the invoice excluding taxes, credits and discounts."""
    subtotal_amount: Optional[int]
    """Represents the object's type."""
    object: Optional[Literal["invoice"]]
    """Date the invoice was issued."""
    issued_at: Optional[str]
    """Amount written off from the invoice."""
    write_off_amount: Optional[int]
    """Items in the invoice."""
    line_items: Optional[List[InvoiceLineItem]]
    """Additional data related to the invoice."""
    metadata: Optional[dict]
    """Paid amount of the invoice."""
    paid_amount: Optional[int]
    """Type of invoice."""
    type: Optional[Literal["adhoc", "subscription_signup", "subscription_renewal", "subscription_update", "order"]]
    """Payment collection method for the invoice."""
    collection_method: Optional[Literal["manual", "automatic", "send_invoice"]]
    """Total amount currently due for this invoice."""
    due_amount: Optional[int]
    """URL to the PDF version of the invoice."""
    download_url: Optional[str]
    """Number of the invoice."""
    number: Optional[str]
    """ID of the object."""
    id: Optional[str]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, List, Literal

from .meter_profile import MeterProfile


class MeterInput(TypedDict, total=False):
    """Name of the meter."""
    name: Optional[str]
    """The name of the event to track usage for."""
    event_name: Optional[str]
    """Meter profiles define aggregation behaviors for a meter."""
    profiles: Optional[List[MeterProfile]]
    """Internal description of the meter."""
    description: Optional[str]
    """Defines the connection between event data and Billingrails accounts."""
    account_mapping: Optional[dict]



# This file is auto-generated. Do not edit manually.

from typing import Dict, Any

from ..types import (
    PaymentInput,
)


class PaymentsResource:
    """Payments resource"""

    def __init__(self, client):
        self.client = client

    def create(self, data: PaymentInput) -> Dict[str, Any]:
        """Create a payment
        
        Create an online or offline payment for an invoice, order, payment request, or credit grant."""
        return self.client.request("POST", f"/payments", json=data)



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, List, Literal


"""Price configuration for a plan or fee."""
class Price(TypedDict, total=False):
    """Flat amount added to percentage model."""
    flat_amount: Optional[int]
    """Currency code."""
    currency: str
    """Pricing tiers (for volume and graduated models)."""
    tiers: Optional[List[dict]]
    """Package size (for package model)."""
    package_size: Optional[int]
    """Amount in currency subunits (for flat, package, or tiered pricing)."""
    amount: Optional[int]
    """Pricing model."""
    model: Literal["flat", "package", "volume", "graduated", "percentage"]
    """Percentage rate (for percentage model)."""
    percentage_rate: Optional[int]



# This file is auto-generated. Do not edit manually.

from typing import Dict, Any

from ..types import (
    EventBatchInput,
    EventInput,
)


class EventsResource:
    """Events resource"""

    def __init__(self, client):
        self.client = client

    def ingest(self, data: EventInput) -> Dict[str, Any]:
        """Ingest event
        
        Ingests an event."""
        return self.client.request("POST", f"/biller/events/ingest", json=data)

    def ingest_batch(self, data: EventBatchInput) -> Dict[str, Any]:
        """Ingest batch events
        
        Ingests batch events."""
        return self.client.request("POST", f"/biller/events/batch", json=data)



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, List

from .credit_grant import CreditGrant


class CreditGrantListResponse(TypedDict, total=False):
    credit_grants: Optional[List[CreditGrant]]
    meta: Optional[dict]



# This file is auto-generated. Do not edit manually.

from typing import Dict, Any

from ..types import (
    FeeCreate,
    FeeListResponse,
    FeeResponse,
    FeeUpdate,
)


class FeesResource:
    """Fees resource"""

    def __init__(self, client):
        self.client = client

    def retrieve(self, id: str, **params) -> FeeResponse:
        """Retrieve fee
        
        Retrieves a fee by ID."""
        return self.client.request("GET", f"/biller/fees/{id}", params=params)

    def update(self, id: str, data: FeeUpdate) -> FeeResponse:
        """Update a fee
        
        Updates a fee."""
        return self.client.request("PUT", f"/biller/fees/{id}", json=data)

    def delete(self, id: str) -> Dict[str, Any]:
        """Delete a fee
        
        Deletes a fee."""
        return self.client.request("DELETE", f"/biller/fees/{id}")

    def list(self) -> FeeListResponse:
        """List fees
        
        Retrieve a list of fees."""
        return self.client.request("GET", f"/biller/fees")

    def create(self, data: FeeCreate) -> FeeResponse:
        """Create a fee
        
        Creates a fee."""
        return self.client.request("POST", f"/biller/fees", json=data)



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, List, Literal


class Error(TypedDict, total=False):
    """Field specific error details."""
    details: Optional[List[dict]]
    type: str
    code: Literal["internal_server_error", "too_many_requests", "method_not_allowed", "invalid_request_url", "unauthorized", "forbidden", "validation_error", "not_found"]
    message: str



# This file is auto-generated. Do not edit manually.

from typing import Dict, Any

from ..types import (
    CreditGrantCreate,
    CreditGrantListResponse,
    CreditGrantResponse,
)


class CreditGrantsResource:
    """Credit grants resource"""

    def __init__(self, client):
        self.client = client

    def list(self) -> CreditGrantListResponse:
        """List credit grants
        
        Retrieves a list of credit grants."""
        return self.client.request("GET", f"/credit_grants")

    def create(self, data: CreditGrantCreate) -> CreditGrantResponse:
        """Create a credit grant
        
        Creates a credit grant."""
        return self.client.request("POST", f"/credit_grants", json=data)

    def retrieve(self, id: str, **params) -> CreditGrantResponse:
        """Retrieve a credit grant
        
        Retrieves a credit grant by ID."""
        return self.client.request("GET", f"/credit_grants/{id}", params=params)

    def reverse_transaction(self, id: str, data: Dict[str, Any]) -> CreditGrantResponse:
        """Reverse credit grant transaction
        
        Reverses a credit grant usage."""
        return self.client.request("POST", f"/credit_grants/{id}/reverse_transaction", json=data)

    def apply(self, id: str, data: Dict[str, Any]) -> CreditGrantResponse:
        """Apply credit grant
        
        Applies a credit grant to an invoice or logs an external usage."""
        return self.client.request("POST", f"/credit_grants/{id}/apply", json=data)

    def expire(self, id: str) -> CreditGrantResponse:
        """Expire credit grant
        
        Expires a credit grant."""
        return self.client.request("POST", f"/credit_grants/{id}/expire", json={})



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, List

from .event_input import EventInput


class EventBatchInput(TypedDict):
    events: List[EventInput]



"""
Billingrails API Client
"""

import requests
from typing import Any

from .resources.accounts import AccountsResource
from .resources.credit_grants import CreditGrantsResource
from .resources.discounts import DiscountsResource
from .resources.events import EventsResource
from .resources.fees import FeesResource
from .resources.invoices import InvoicesResource
from .resources.meters import MetersResource
from .resources.orders import OrdersResource
from .resources.payment_pages import PaymentPagesResource
from .resources.payments import PaymentsResource
from .resources.plans import PlansResource
from .resources.products import ProductsResource
from .resources.subscriptions import SubscriptionsResource

DEFAULT_TIMEOUT = 30
DEFAULT_MAX_RETRIES = 3


class BillerNamespace:
    """Biller-specific resources"""
    
    def __init__(self, client: "Billingrails"):
        self.events = EventsResource(client)
        self.meters = MetersResource(client)
        self.plans = PlansResource(client)
        self.subscriptions = SubscriptionsResource(client)


class SellerNamespace:
    """Seller-specific resources"""
    
    def __init__(self, client: "Billingrails"):
        self.products = ProductsResource(client)
        self.orders = OrdersResource(client)


class Billingrails:
    """Billingrails API client with nested resource namespaces"""

    def __init__(
        self,
        api_key: str,
        base_url: str = "https://api.billingrails.com",
        timeout: int = DEFAULT_TIMEOUT,
        max_retries: int = DEFAULT_MAX_RETRIES
    ):
        self.session = requests.Session()
        self.session.headers.update({
            "Content-Type": "application/json",
            "Authorization": f"Bearer {api_key}",
        })
        self.base_url = base_url.rstrip("/")
        self.timeout = timeout
        self.max_retries = max_retries

        # Top-level resources
        self.accounts = AccountsResource(self)
        self.credit_grants = CreditGrantsResource(self)
        self.discounts = DiscountsResource(self)
        self.fees = FeesResource(self)
        self.invoices = InvoicesResource(self)
        self.payments = PaymentsResource(self)
        self.payment_pages = PaymentPagesResource(self)

        # Nested namespaces
        self.biller = BillerNamespace(self)
        self.seller = SellerNamespace(self)

    def request(self, method: str, path: str, **kwargs) -> Any:
        """Make an HTTP request with retry logic"""
        url = f"{self.base_url}{path}"
        if 'timeout' not in kwargs:
            kwargs['timeout'] = self.timeout

        last_exception = None
        for attempt in range(self.max_retries):
            try:
                response = self.session.request(method, url, **kwargs)
                response.raise_for_status()
                return response.json()
            except requests.exceptions.RequestException as e:
                last_exception = e
                if attempt < self.max_retries - 1:
                    # Exponential backoff
                    import time
                    time.sleep(2 ** attempt)
                    continue
                raise

        if last_exception:
            raise last_exception



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, List

from .plan import Plan


class PlanListResponse(TypedDict, total=False):
    meta: Optional[dict]
    plans: Optional[List[Plan]]



# This file is auto-generated. Do not edit manually.

from .base import ListResponseMeta, ApiErrorDetails, ApiError

from .account import Account
from .account_balances_response import AccountBalancesResponse
from .account_create import AccountCreate
from .account_debit import AccountDebit
from .account_debit_response import AccountDebitResponse
from .account_input import AccountInput
from .account_list_response import AccountListResponse
from .account_response import AccountResponse
from .account_update import AccountUpdate
from .address import Address
from .credit_grant import CreditGrant
from .credit_grant_create import CreditGrantCreate
from .credit_grant_list_response import CreditGrantListResponse
from .credit_grant_pay_response import CreditGrantPayResponse
from .credit_grant_response import CreditGrantResponse
from .discount import Discount
from .discount_create import DiscountCreate
from .discount_input import DiscountInput
from .discount_list_response import DiscountListResponse
from .discount_response import DiscountResponse
from .discount_update import DiscountUpdate
from .error import Error
from .event import Event
from .event_batch_input import EventBatchInput
from .event_batch_response import EventBatchResponse
from .event_input import EventInput
from .event_response import EventResponse
from .fee import Fee
from .fee_create import FeeCreate
from .fee_input import FeeInput
from .fee_list_response import FeeListResponse
from .fee_response import FeeResponse
from .fee_update import FeeUpdate
from .interval import Interval
from .invoice import Invoice
from .invoice_create import InvoiceCreate
from .invoice_input import InvoiceInput
from .invoice_line_item import InvoiceLineItem
from .invoice_list_response import InvoiceListResponse
from .invoice_response import InvoiceResponse
from .invoice_update import InvoiceUpdate
from .meter import Meter
from .meter_create import MeterCreate
from .meter_input import MeterInput
from .meter_list_response import MeterListResponse
from .meter_profile import MeterProfile
from .meter_response import MeterResponse
from .meter_update import MeterUpdate
from .order import Order
from .order_create import OrderCreate
from .order_input import OrderInput
from .order_list_response import OrderListResponse
from .order_response import OrderResponse
from .order_update import OrderUpdate
from .payment import Payment
from .payment_input import PaymentInput
from .payment_method import PaymentMethod
from .payment_page import PaymentPage
from .payment_page_create import PaymentPageCreate
from .payment_page_input import PaymentPageInput
from .payment_page_list_response import PaymentPageListResponse
from .payment_page_response import PaymentPageResponse
from .payment_page_update import PaymentPageUpdate
from .payment_request import PaymentRequest
from .plan import Plan
from .plan_create import PlanCreate
from .plan_input import PlanInput
from .plan_list_response import PlanListResponse
from .plan_response import PlanResponse
from .plan_update import PlanUpdate
from .price import Price
from .product import Product
from .product_create import ProductCreate
from .product_input import ProductInput
from .product_list_response import ProductListResponse
from .product_response import ProductResponse
from .product_update import ProductUpdate
from .subscription import Subscription
from .subscription_create import SubscriptionCreate
from .subscription_input import SubscriptionInput
from .subscription_list_response import SubscriptionListResponse
from .subscription_response import SubscriptionResponse
from .subscription_update import SubscriptionUpdate

__all__ = [
    "ListResponseMeta",
    "ApiErrorDetails",
    "ApiError",
    "Account",
    "AccountBalancesResponse",
    "AccountCreate",
    "AccountDebit",
    "AccountDebitResponse",
    "AccountInput",
    "AccountListResponse",
    "AccountResponse",
    "AccountUpdate",
    "Address",
    "CreditGrant",
    "CreditGrantCreate",
    "CreditGrantListResponse",
    "CreditGrantPayResponse",
    "CreditGrantResponse",
    "Discount",
    "DiscountCreate",
    "DiscountInput",
    "DiscountListResponse",
    "DiscountResponse",
    "DiscountUpdate",
    "Error",
    "Event",
    "EventBatchInput",
    "EventBatchResponse",
    "EventInput",
    "EventResponse",
    "Fee",
    "FeeCreate",
    "FeeInput",
    "FeeListResponse",
    "FeeResponse",
    "FeeUpdate",
    "Interval",
    "Invoice",
    "InvoiceCreate",
    "InvoiceInput",
    "InvoiceLineItem",
    "InvoiceListResponse",
    "InvoiceResponse",
    "InvoiceUpdate",
    "Meter",
    "MeterCreate",
    "MeterInput",
    "MeterListResponse",
    "MeterProfile",
    "MeterResponse",
    "MeterUpdate",
    "Order",
    "OrderCreate",
    "OrderInput",
    "OrderListResponse",
    "OrderResponse",
    "OrderUpdate",
    "Payment",
    "PaymentInput",
    "PaymentMethod",
    "PaymentPage",
    "PaymentPageCreate",
    "PaymentPageInput",
    "PaymentPageListResponse",
    "PaymentPageResponse",
    "PaymentPageUpdate",
    "PaymentRequest",
    "Plan",
    "PlanCreate",
    "PlanInput",
    "PlanListResponse",
    "PlanResponse",
    "PlanUpdate",
    "Price",
    "Product",
    "ProductCreate",
    "ProductInput",
    "ProductListResponse",
    "ProductResponse",
    "ProductUpdate",
    "Subscription",
    "SubscriptionCreate",
    "SubscriptionInput",
    "SubscriptionListResponse",
    "SubscriptionResponse",
    "SubscriptionUpdate",
]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional


class AccountDebit(TypedDict, total=False):
    """The asset code for the debit."""
    asset_code: Optional[str]
    """The currency code for the debit."""
    currency: Optional[str]
    """Optional reference ID for tracking the debit."""
    reference_id: Optional[str]
    """The amount to debit in the smallest currency unit."""
    amount: int



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, Literal

from .payment_method import PaymentMethod


class Payment(TypedDict, total=False):
    payment_method: Optional[PaymentMethod]
    """Represents the object's type."""
    object: Optional[Literal["payment"]]
    """Amount of the payment."""
    amount: Optional[int]
    """ID of the account the payment belongs to."""
    account_id: Optional[str]
    """ID of the payment."""
    id: Optional[str]
    """Date the payment was created."""
    created_at: Optional[str]
    """Reference for the payment."""
    reference_id: Optional[str]
    """Status of the payment."""
    status: Optional[Literal["pending", "succeeded", "failed", "canceled"]]
    """Description of the payment."""
    description: Optional[str]
    """ID of the payment method used."""
    payment_method_id: Optional[str]
    """Currency of the payment."""
    currency: Optional[str]



# This file is auto-generated. Do not edit manually.

from typing import Dict, Any

from ..types import (
    InvoiceCreate,
    InvoiceListResponse,
    InvoiceResponse,
    InvoiceUpdate,
)


class InvoicesResource:
    """Invoices resource"""

    def __init__(self, client):
        self.client = client

    def issue(self, id: str, data: Dict[str, Any]) -> Dict[str, Any]:
        """Issue an invoice
        
        Issues an invoice."""
        return self.client.request("POST", f"/invoices/{id}/issue", json=data)

    def retrieve(self, id: str, **params) -> InvoiceResponse:
        """Retrieve invoice
        
        Retrieves an invoice by ID."""
        return self.client.request("GET", f"/invoices/{id}", params=params)

    def update(self, id: str, data: InvoiceUpdate) -> InvoiceResponse:
        """Update an invoice
        
        Updates an invoice."""
        return self.client.request("PUT", f"/invoices/{id}", json=data)

    def list(self, **params) -> InvoiceListResponse:
        """List invoices
        
        Retrieves a list of invoices."""
        return self.client.request("GET", f"/invoices", params=params)

    def create(self, data: InvoiceCreate) -> InvoiceResponse:
        """Create an invoice
        
        Creates an invoice."""
        return self.client.request("POST", f"/invoices", json=data)



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, Literal


"""Billing interval definition."""
class Interval(TypedDict):
    """Frequency of the interval."""
    frequency: int
    """Unit of the interval."""
    unit: Literal["day", "week", "month", "year"]



# This file is auto-generated. Do not edit manually.

from ..types import (
    PlanCreate,
    PlanListResponse,
    PlanResponse,
    PlanUpdate,
)


class PlansResource:
    """Plans resource"""

    def __init__(self, client):
        self.client = client

    def list(self, **params) -> PlanListResponse:
        """List plans
        
        Retrieves a list of plans."""
        return self.client.request("GET", f"/biller/plans", params=params)

    def create(self, data: PlanCreate) -> PlanResponse:
        """Create a plan
        
        Creates a plan."""
        return self.client.request("POST", f"/biller/plans", json=data)

    def retrieve(self, id: str, **params) -> PlanResponse:
        """Retrieve a plan
        
        Retrieves plan by ID."""
        return self.client.request("GET", f"/biller/plans/{id}", params=params)

    def update(self, id: str, data: PlanUpdate) -> PlanResponse:
        """Update a plan
        
        Updates a plan."""
        return self.client.request("PUT", f"/biller/plans/{id}", json=data)



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, Literal

from .address import Address


class Account(TypedDict, total=False):
    """Represents the object's type."""
    object: Optional[Literal["account"]]
    """ID of the object."""
    id: Optional[str]
    """Type of account."""
    type: Optional[Literal["individual", "organization"]]
    """Invoice settings for the account."""
    invoice_settings: Optional[dict]
    """Additional data related to the account."""
    metadata: Optional[dict]
    """Name of the account."""
    name: Optional[str]
    """Timezone for the account."""
    timezone: Optional[str]
    """Country of the account."""
    country: Optional[str]
    """Timestamp indicating when the object was created."""
    created_at: Optional[str]
    """Default currency of the account."""
    default_currency: Optional[str]
    """External ID of the object."""
    external_id: Optional[str]
    """Shipping address of the account."""
    shipping_address: Optional[Address]
    """Billing address of the account."""
    billing_address: Optional[Address]
    """Email of the account."""
    email: Optional[str]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, List

from .order import Order


class OrderListResponse(TypedDict, total=False):
    orders: Optional[List[Order]]
    meta: Optional[dict]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, List


class ListResponseMeta(TypedDict):
    """Pagination metadata"""
    page: int
    page_size: int
    total_pages: int
    total_count: int


class ApiErrorDetails(TypedDict, total=False):
    """API error details"""
    field: Optional[str]
    message: Optional[str]


class ApiError(TypedDict):
    """API error response"""
    type: str
    code: str
    message: str
    details: Optional[List[ApiErrorDetails]]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional


class EventInput(TypedDict, total=False):
    """Properties associated with the event."""
    properties: dict
    """The name of the event."""
    event_name: str
    """Unique identifier for this event."""
    idempotency_key: Optional[str]
    """Timestamp indicating the occurrence of the event."""
    timestamp: Optional[str]



# This file is auto-generated. Do not edit manually.

from typing import Dict, Any

from ..types import (
    OrderCreate,
    OrderListResponse,
    OrderResponse,
    OrderUpdate,
)


class OrdersResource:
    """Orders resource"""

    def __init__(self, client):
        self.client = client

    def list(self, **params) -> OrderListResponse:
        """List orders
        
        Retrieves a list of orders."""
        return self.client.request("GET", f"/seller/orders", params=params)

    def create(self, data: OrderCreate) -> OrderResponse:
        """Create an order
        
        Creates an order."""
        return self.client.request("POST", f"/seller/orders", json=data)

    def retrieve(self, id: str, **params) -> OrderResponse:
        """Retrieve an order
        
        Retrieves an order by ID."""
        return self.client.request("GET", f"/seller/orders/{id}", params=params)

    def update(self, id: str, data: OrderUpdate) -> OrderResponse:
        """Update an order
        
        Updates an order."""
        return self.client.request("PUT", f"/seller/orders/{id}", json=data)

    def cancel(self, id: str) -> Dict[str, Any]:
        """Cancel an order
        
        Cancels an order."""
        return self.client.request("POST", f"/seller/orders/{id}/cancel", json={})



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, Literal


class PaymentMethod(TypedDict, total=False):
    """Type of the payment method."""
    type: Optional[str]
    """Represents the object's type."""
    object: Optional[Literal["payment_method"]]
    """Name of the account associated with the payment method."""
    account_name: Optional[str]
    """ID of the payment method."""
    id: Optional[str]
    """Details of the payment method."""
    details: Optional[dict]
    """Date the payment method was created."""
    created_at: Optional[str]
    """Payment provider for the payment method. ex. `stripe`, `paystack`."""
    provider: Optional[str]
    """Status of the payment method."""
    status: Optional[str]
    """ID of the account associated with the payment method."""
    account_id: Optional[str]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, List

from .meter import Meter


class MeterListResponse(TypedDict, total=False):
    meta: Optional[dict]
    meters: Optional[List[Meter]]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, List

from .subscription import Subscription


class SubscriptionListResponse(TypedDict, total=False):
    meta: Optional[dict]
    subscriptions: Optional[List[Subscription]]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional

from .account import Account


class AccountResponse(TypedDict, total=False):
    meta: Optional[dict]
    account: Optional[Account]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, List

from .event import Event


class EventBatchResponse(TypedDict, total=False):
    meta: Optional[dict]
    events: Optional[List[Event]]



# This file is auto-generated. Do not edit manually.

from ..types import (
    MeterCreate,
    MeterListResponse,
    MeterResponse,
    MeterUpdate,
)


class MetersResource:
    """Meters resource"""

    def __init__(self, client):
        self.client = client

    def retrieve(self, id: str, **params) -> MeterResponse:
        """Retrieve a meter
        
        Retrieves meter by ID."""
        return self.client.request("GET", f"/biller/meters/{id}", params=params)

    def update(self, id: str, data: MeterUpdate) -> MeterResponse:
        """Update a meter
        
        Updates a meter."""
        return self.client.request("PUT", f"/biller/meters/{id}", json=data)

    def list(self, **params) -> MeterListResponse:
        """List meters
        
        Retrieves a list of meters."""
        return self.client.request("GET", f"/biller/meters", params=params)

    def create(self, data: MeterCreate) -> MeterResponse:
        """Create a meter
        
        Creates a meter."""
        return self.client.request("POST", f"/biller/meters", json=data)



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, List, Literal

from .fee import Fee
from .price import Price
from .interval import Interval


class Plan(TypedDict, total=False):
    """Meter ID associated with the fee (for usage billing)."""
    meter_id: Optional[str]
    """Billing cycles of the fee."""
    billing_cycles: Optional[int]
    """Description of the plan."""
    description: Optional[str]
    """Represents the object's type."""
    object: Optional[Literal["plan"]]
    """ID of the object."""
    id: Optional[str]
    """Number of free units allowed for this fee."""
    free_units: Optional[int]
    """Name that appears on invoices for this plan."""
    invoice_name: Optional[str]
    """Meter name (for usage billing)."""
    meter_name: Optional[str]
    """Name of the plan."""
    name: Optional[str]
    """Timestamp indicating when the object was created."""
    created_at: Optional[str]
    """Price for the plan."""
    price: Optional[Price]
    """Trial period of the plan in days."""
    trial_period_days: Optional[int]
    """Determines whether fee is billed in advance or arrears."""
    bill_timing: Optional[Literal["advance", "arrears"]]
    """Meter profile name (for usage billing)."""
    meter_profile_name: Optional[str]
    """Fees associated with the plan."""
    fees: Optional[List[Fee]]
    """Internal identifier of the plan."""
    code: Optional[str]
    """Billing interval for the plan."""
    interval: Optional[Interval]
    """Meter profile ID to use for this fee (for usage billing)."""
    meter_profile_id: Optional[str]
    """Status of the plan."""
    status: Optional[Literal["draft", "active", "archived"]]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, Literal


class InvoiceLineItem(TypedDict, total=False):
    """Unit amount of the invoice item."""
    unit_amount: Optional[int]
    """End date for the billing period."""
    billing_end: Optional[str]
    """Represents the object's type."""
    object: Optional[Literal["invoice_line_item"]]
    """Quantity of the invoice item."""
    quantity: Optional[int]
    """Description of the invoice item."""
    description: Optional[str]
    """Name of the invoice item."""
    name: Optional[str]
    """ID of the invoice item."""
    id: Optional[str]
    """Total amount of the invoice item."""
    total_amount: Optional[int]
    """Subtotal amount of the invoice item."""
    subtotal_amount: Optional[int]
    """Start date for the billing period."""
    billing_start: Optional[str]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, List, Literal


class CreditGrantCreate(TypedDict, total=False):
    """Currency of the credit grant. Required when `asset_code` is not set."""
    currency: Optional[str]
    """Asset code of the credit grant. Required when `currency` is not set."""
    asset_code: Optional[str]
    """Date when the credit grant expires."""
    expires_at: Optional[str]
    """Whether to create a payment link for the credit grant. Requires `integration_id`."""
    with_payment_link: Optional[bool]
    """Name of the credit grant."""
    name: Optional[str]
    """Integration ID for payment processing."""
    integration_id: Optional[str]
    """Payment methods to show on the payment page."""
    allowed_payment_methods: Optional[List[str]]
    """Whether to create an invoice after payment. Defaults to `true`."""
    invoice_after_payment: Optional[bool]
    """Additional data related to the credit grant."""
    metadata: Optional[dict]
    """URL to redirect after payment."""
    return_url: Optional[str]
    """Type of credit grant."""
    type: Literal["paid", "promotional"]
    """Account ID associated with the credit grant."""
    account_id: str
    """Description of the credit grant."""
    description: Optional[str]
    """Granted amount in currency subunits or quantity of credit assets."""
    grant_amount: int



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, List

from .address import Address


class OrderInput(TypedDict, total=False):
    """Shipping address of the order."""
    shipping_address: Optional[Address]
    """Additional data related to the order."""
    metadata: Optional[dict]
    """Currency of the order."""
    currency: Optional[str]
    """Whether to use billing address for shipping."""
    use_billing_address_for_shipping: Optional[bool]
    """Line items in the order."""
    line_items: Optional[List[dict]]
    """Billing address of the order."""
    billing_address: Optional[Address]
    """Account ID associated with the order."""
    account_id: Optional[str]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, Literal

from .interval import Interval
from .price import Price


class PlanUpdate(TypedDict, total=False):
    """Trial period of the plan in days."""
    trial_period_days: Optional[int]
    """Number of free units included."""
    free_units: Optional[int]
    """Billing interval for the plan."""
    interval: Optional[Interval]
    """Number of billing cycles for this fee."""
    billing_cycles: Optional[int]
    """Name that appears on invoices for this plan."""
    invoice_name: Optional[str]
    """Description of the plan."""
    description: Optional[str]
    """Name of the plan."""
    name: Optional[str]
    """When to bill for this fee."""
    bill_timing: Optional[Literal["advance", "arrears"]]
    """Price object for the plan."""
    price: Optional[Price]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional

from .subscription import Subscription


class SubscriptionResponse(TypedDict, total=False):
    meta: Optional[dict]
    subscription: Optional[Subscription]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, List

from .invoice import Invoice


class InvoiceListResponse(TypedDict, total=False):
    meta: Optional[dict]
    invoices: Optional[List[Invoice]]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, List

from .account import Account


class AccountListResponse(TypedDict, total=False):
    meta: Optional[dict]
    accounts: Optional[List[Account]]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, List

from .payment_page import PaymentPage


class PaymentPageListResponse(TypedDict, total=False):
    payment_pages: Optional[List[PaymentPage]]
    meta: Optional[dict]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, List, Literal


class InvoiceCreate(TypedDict, total=False):
    """Items in the invoice."""
    line_items: List[dict]
    """Payment collection method for the invoice."""
    collection_method: Literal["automatic", "manual"]
    """Currency of the invoice."""
    currency: str
    """Account ID associated with the invoice."""
    account_id: str
    """Payment method ID to charge (required if collection_method is automatic)."""
    payment_method_id: Optional[str]
    """Date the invoice is due."""
    due_at: str



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional

from .meter import Meter


class MeterResponse(TypedDict, total=False):
    meter: Optional[Meter]
    meta: Optional[dict]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, List, Literal


class Order(TypedDict, total=False):
    """ID of the object."""
    id: Optional[str]
    """Subtotal amount of the order."""
    subtotal_amount: Optional[int]
    """Timestamp indicating when the object was created."""
    created_at: Optional[str]
    """Paid amount of the order."""
    paid_amount: Optional[int]
    """Order number."""
    number: Optional[str]
    """Line items in the order."""
    line_items: Optional[List[dict]]
    """Payment status of the order."""
    payment_status: Optional[Literal["pending", "paid", "partially_paid", "refunded", "failed"]]
    """Return status of the order."""
    return_status: Optional[Literal["none", "requested", "approved", "returned"]]
    """Additional data related to the order."""
    metadata: Optional[dict]
    """Represents the object's type."""
    object: Optional[Literal["order"]]
    """Status of the order."""
    status: Optional[Literal["draft", "pending", "confirmed", "fulfilled", "canceled"]]
    """Outstanding amount of the order."""
    outstanding_amount: Optional[int]
    """Total amount of the order."""
    total_amount: Optional[int]
    """Fulfillment status of the order."""
    fulfillment_status: Optional[Literal["pending", "fulfilled", "partially_fulfilled", "canceled"]]
    """Account ID associated with the order."""
    account_id: Optional[str]
    """Currency of the order."""
    currency: Optional[str]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional

from .discount import Discount


class DiscountResponse(TypedDict, total=False):
    discount: Optional[Discount]
    meta: Optional[dict]



# This file is auto-generated. Do not edit manually.

from ..types import (
    PaymentPageCreate,
    PaymentPageListResponse,
    PaymentPageResponse,
    PaymentPageUpdate,
)


class PaymentPagesResource:
    """Payment pages resource"""

    def __init__(self, client):
        self.client = client

    def list(self, **params) -> PaymentPageListResponse:
        """List payment pages
        
        Retrieves a list of payment pages."""
        return self.client.request("GET", f"/payment_pages", params=params)

    def create(self, data: PaymentPageCreate) -> PaymentPageResponse:
        """Create payment page
        
        Creates a new payment page."""
        return self.client.request("POST", f"/payment_pages", json=data)

    def retrieve(self, id: str, **params) -> PaymentPageResponse:
        """Retrieve payment page
        
        Retrieves a payment page by ID."""
        return self.client.request("GET", f"/payment_pages/{id}", params=params)

    def update(self, id: str, data: PaymentPageUpdate) -> PaymentPageResponse:
        """Update payment page
        
        Updates a payment page."""
        return self.client.request("PUT", f"/payment_pages/{id}", json=data)



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, List, Literal


class SubscriptionInput(TypedDict, total=False):
    """Modify fee quantities for the subscription plans and fees."""
    items: Optional[List[dict]]
    """Plan ID associated with the subscription."""
    plan_id: Optional[str]
    """Billing cycle type of the subscription."""
    billing_cycle_type: Optional[Literal["anniversary", "calendar"]]
    """Payment method ID associated with the subscription. Required if collection_method is `automatic`."""
    payment_method_id: Optional[str]
    """Trial period of the subscription in days. Overrides the plan's trial period if set."""
    trial_period_days: Optional[int]
    """Payment collection method for invoices generated by the subscription."""
    collection_method: Optional[str]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional

from .payment_page import PaymentPage


class PaymentPageResponse(TypedDict, total=False):
    payment_page: Optional[PaymentPage]
    meta: Optional[dict]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, Literal


class Product(TypedDict, total=False):
    """Status of the product."""
    status: Optional[Literal["active", "inactive"]]
    """Amount of the product in cents."""
    amount: Optional[int]
    """ID of the object."""
    id: Optional[str]
    """Model of the product."""
    model: Optional[str]
    """SKU of the product."""
    sku: Optional[str]
    """Vendor of the product."""
    vendor: Optional[str]
    """Unique code of the product."""
    code: Optional[str]
    """Represents the object's type."""
    object: Optional[Literal["product"]]
    """Currency of the product."""
    currency: Optional[str]
    """Description of the product."""
    description: Optional[str]
    """Name of the product."""
    name: Optional[str]
    """Timestamp indicating when the object was created."""
    created_at: Optional[str]
    """URL of the product image."""
    image_url: Optional[str]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional

from .invoice import Invoice


class InvoiceResponse(TypedDict, total=False):
    invoice: Optional[Invoice]
    meta: Optional[dict]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, List

from .fee import Fee


class FeeListResponse(TypedDict, total=False):
    fees: Optional[List[Fee]]
    meta: Optional[dict]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, Literal


class Event(TypedDict, total=False):
    """ID of the object."""
    id: Optional[str]
    """The name of the event."""
    event_name: Optional[str]
    """Represents the object's type."""
    object: Optional[Literal["event"]]
    """Properties associated with the event."""
    properties: Optional[dict]
    """Unique identifier for this event."""
    idempotency_key: Optional[str]
    """Timestamp indicating the occurrence of the event."""
    timestamp: Optional[str]



# This file is auto-generated. Do not edit manually.

from ..types import (
    DiscountCreate,
    DiscountListResponse,
    DiscountResponse,
    DiscountUpdate,
)


class DiscountsResource:
    """Discounts resource"""

    def __init__(self, client):
        self.client = client

    def retrieve(self, id: str, **params) -> DiscountResponse:
        """Retrieve discount
        
        Retrieves a discount by ID."""
        return self.client.request("GET", f"/discounts/{id}", params=params)

    def update(self, id: str, data: DiscountUpdate) -> DiscountResponse:
        """Update a discount
        
        Updates a discount."""
        return self.client.request("PUT", f"/discounts/{id}", json=data)

    def list(self, **params) -> DiscountListResponse:
        """List discounts
        
        Retrieve a list of discounts."""
        return self.client.request("GET", f"/discounts", params=params)

    def create(self, data: DiscountCreate) -> DiscountResponse:
        """Create a discount
        
        Creates a discount."""
        return self.client.request("POST", f"/discounts", json=data)



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, Literal


class DiscountCreate(TypedDict, total=False):
    """Description of the discount."""
    description: Optional[str]
    """Amount off in currency subunits. Required when type is `fixed_amount`."""
    amount_off: Optional[int]
    """Maximum number of billing periods discount can recur (null = forever)."""
    max_recurring_intervals: Optional[int]
    """Type of discount."""
    type: Literal["percentage", "fixed_amount"]
    """Maximum number of redemptions across all accounts."""
    max_redemptions: Optional[int]
    """Date when the discount becomes valid."""
    valid_from: Optional[str]
    """Percentage off. Required when type is `percentage`."""
    percent_off: Optional[int]
    """Name of the discount."""
    name: str
    """Whether discount can be applied to multiple billing periods."""
    recurring: Optional[bool]
    """Unique code for the discount."""
    code: str
    """Maximum number of redemptions per account."""
    max_redemptions_per_account: Optional[int]
    """Date when the discount expires."""
    valid_until: Optional[str]
    """Additional data related to the discount."""
    metadata: Optional[dict]
    """Currency. Required when type is `fixed_amount`."""
    currency: Optional[str]



# This file is auto-generated. Do not edit manually.

from typing import Dict, Any

from ..types import (
    AccountBalancesResponse,
    AccountCreate,
    AccountDebit,
    AccountDebitResponse,
    AccountListResponse,
    AccountResponse,
    AccountUpdate,
)


class AccountsResource:
    """Accounts resource"""

    def __init__(self, client):
        self.client = client

    def retrieve(self, id: str, **params) -> AccountResponse:
        """Retrieve an account
        
        Retrieves an account by ID."""
        return self.client.request("GET", f"/accounts/{id}", params=params)

    def update(self, id: str, data: AccountUpdate) -> AccountResponse:
        """Update an account
        
        Updates an account."""
        return self.client.request("PUT", f"/accounts/{id}", json=data)

    def get_balances(self, id: str, **params) -> AccountBalancesResponse:
        """Get balances
        
        Retrieve credit balances for an account."""
        return self.client.request("GET", f"/accounts/{id}/balances", params=params)

    def debit(self, id: str, data: AccountDebit) -> AccountDebitResponse:
        """Debit balance
        
        Debits an account's balance."""
        return self.client.request("POST", f"/accounts/{id}/debit", json=data)

    def list(self, **params) -> AccountListResponse:
        """List accounts
        
        Retrieve a list of accounts."""
        return self.client.request("GET", f"/accounts", params=params)

    def create(self, data: AccountCreate) -> AccountResponse:
        """Create an account
        
        Creates an account."""
        return self.client.request("POST", f"/accounts", json=data)



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional

from .fee import Fee


class FeeResponse(TypedDict, total=False):
    meta: Optional[dict]
    fee: Optional[Fee]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional

from .credit_grant import CreditGrant


class CreditGrantResponse(TypedDict, total=False):
    meta: Optional[dict]
    credit_grant: Optional[CreditGrant]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional

from .product import Product


class ProductResponse(TypedDict, total=False):
    product: Optional[Product]
    meta: Optional[dict]



# This file is auto-generated. Do not edit manually.

from ..types import (
    SubscriptionCreate,
    SubscriptionListResponse,
    SubscriptionResponse,
)


class SubscriptionsResource:
    """Subscriptions resource"""

    def __init__(self, client):
        self.client = client

    def list(self, **params) -> SubscriptionListResponse:
        """List subscriptions
        
        Retrieves a list of subscriptions."""
        return self.client.request("GET", f"/biller/subscriptions", params=params)

    def create(self, data: SubscriptionCreate) -> SubscriptionResponse:
        """Create a subscription
        
        Creates a subscription."""
        return self.client.request("POST", f"/biller/subscriptions", json=data)

    def retrieve(self, id: str, **params) -> SubscriptionResponse:
        """Retrieve subscription
        
        Retrieves a subscription by ID."""
        return self.client.request("GET", f"/biller/subscriptions/{id}", params=params)

    def update(self, id: str, data: SubscriptionCreate) -> SubscriptionResponse:
        """Update a subscription
        
        Updates a subscription."""
        return self.client.request("PUT", f"/biller/subscriptions/{id}", json=data)



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, List, Literal


class InvoiceUpdate(TypedDict, total=False):
    """Payment method ID to charge (required if collection_method is automatic)."""
    payment_method_id: Optional[str]
    """Date the invoice is due."""
    due_at: Optional[str]
    """Payment collection method for the invoice."""
    collection_method: Optional[Literal["automatic", "manual"]]
    line_items: Optional[List[dict]]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, List


class AccountBalancesResponse(TypedDict, total=False):
    account_id: Optional[str]
    """Balances by currency."""
    balances: Optional[List[dict]]
    meta: Optional[dict]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, List, Literal


class MeterProfile(TypedDict, total=False):
    """Specifies the method for aggregating matching events."""
    aggregation_method: Literal["sum", "count", "count_unique", "min", "max", "latest"]
    """ID of the profile."""
    id: Optional[str]
    """A key that specifies which event property is used to aggregate data. Not required for `count` aggregation method."""
    aggregation_property: Optional[str]
    """Name of the meter profile."""
    name: str
    """Code identifier of the meter profile."""
    code: str
    """Time window for aggregation."""
    aggregation_time_window: Optional[Literal["hour", "day"]]
    """Whether the profile tracks recurring usage."""
    recurring: bool
    """Filters to apply to events for this profile."""
    filters: Optional[List[dict]]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional


class CreditGrantPayResponse(TypedDict, total=False):
    meta: Optional[dict]
    credit_grant_id: Optional[str]
    payment_link: Optional[str]



from setuptools import setup, find_packages

setup(
    name="billingrails",
    version="0.1.0",
    description="Official Python SDK for Billingrails API - Flexible, composable, and intuitive API-first commerce platform",
    author="Billingrails",
    packages=find_packages(),
    install_requires=[
        "requests>=2.25.0",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",
)



# This file is auto-generated. Do not edit manually.

from .invoices import InvoicesResource
from .payments import PaymentsResource
from .discounts import DiscountsResource
from .credit_grants import CreditGrantsResource
from .plans import PlansResource
from .payment_pages import PaymentPagesResource
from .products import ProductsResource
from .subscriptions import SubscriptionsResource
from .meters import MetersResource
from .events import EventsResource
from .orders import OrdersResource
from .accounts import AccountsResource
from .fees import FeesResource

__all__ = [
    "InvoicesResource",
    "PaymentsResource",
    "DiscountsResource",
    "CreditGrantsResource",
    "PlansResource",
    "PaymentPagesResource",
    "ProductsResource",
    "SubscriptionsResource",
    "MetersResource",
    "EventsResource",
    "OrdersResource",
    "AccountsResource",
    "FeesResource",
]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, Literal


class PaymentInput(TypedDict, total=False):
    """ID of the order to create payment for. Required when `invoice_id`, `payment_request_id` and `credit_grant_id` are not provided."""
    order_id: Optional[str]
    """ID of the invoice to create payment for. Required when `payment_request_id`, `order_id` and `credit_grant_id` are not provided."""
    invoice_id: Optional[str]
    """ID of the payment method to use for the payment. Payment method will be charged immediately. Required for online payments when `integration_id` is not provided."""
    payment_method_id: Optional[str]
    """For offline payments only. Timestamp when the payment succeeded. Defaults to current time if not provided."""
    succeeded_at: Optional[str]
    """ID of the payment request to create payment for. Required when `invoice_id`, `order_id` and `credit_grant_id` are not provided."""
    payment_request_id: Optional[str]
    """Reference for the payment."""
    reference_id: str
    """ID of the credit grant to create payment for. Required when `invoice_id`, `payment_request_id` and `order_id` are not provided."""
    credit_grant_id: Optional[str]
    """ID of the payment integration to process the payment through. Required for online payments when `payment_method_id` is not provided."""
    integration_id: Optional[str]
    """Mode of the payment. Offline payments record manual payments made outside of the system. Online payments process payments through a payment provider."""
    mode: Literal["online", "offline"]
    """ID of the account to record payment for."""
    account_id: str



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, Literal


class CreditGrant(TypedDict, total=False):
    """Date when the credit grant expires."""
    expires_at: Optional[str]
    """Status of the credit grant."""
    status: Optional[Literal["pending", "granted", "depleted", "expired", "voided"]]
    """Granted amount in cents or quantity of credit assets."""
    grant_amount: Optional[int]
    """Timestamp indicating when the object was created."""
    created_at: Optional[str]
    """Payment status of the credit grant."""
    payment_status: Optional[Literal["comped", "paid", "unpaid"]]
    """ID of the object."""
    id: Optional[str]
    """Type of credit grant."""
    type: Optional[Literal["paid", "promotional", "refund"]]
    """Available amount in cents or quantity of credit assets."""
    available_amount: Optional[int]
    """Currency of the credit grant."""
    currency: Optional[str]
    """Account ID associated with the credit grant."""
    account_id: Optional[str]
    """Date when the credit grant becomes effective."""
    effective_at: Optional[str]
    """Represents the object's type."""
    object: Optional[Literal["credit_grant"]]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, Literal


class ProductUpdate(TypedDict, total=False):
    """Name of the product."""
    name: Optional[str]
    """Unique code of the product."""
    code: Optional[str]
    """Description of the product."""
    description: Optional[str]
    """Model of the product."""
    model: Optional[str]
    """Currency of the product."""
    currency: Optional[str]
    """Status of the product."""
    status: Optional[Literal["active", "inactive"]]
    """URL of the product image."""
    image_url: Optional[str]
    """SKU of the product."""
    sku: Optional[str]
    """Amount of the product in cents."""
    amount: Optional[int]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional

from .event import Event


class EventResponse(TypedDict, total=False):
    meta: Optional[dict]
    event: Optional[Event]



# This file is auto-generated. Do not edit manually.

from typing import TypedDict, Optional, List

from .discount import Discount


class DiscountListResponse(TypedDict, total=False):
    meta: Optional[dict]
    discounts: Optional[List[Discount]]
